from trac.core import *
from genshi.core import Markup
from trac.wiki.macros import WikiMacroBase
from trac.wiki.model import WikiPage
from trac.wiki import Formatter
from StringIO import StringIO

# This borrows heavily from TOCMacro, which is copyright Alec Thomas.
# TOCMacro is available under the BSD License from:
# http://trac-hacks.org/wiki/TocMacro
class CatPages(WikiMacroBase):
    """
    Concatenate a set of wiki pages and display them in place.
    {{{
        [[CatPages(Page1, Page2, Page3)]]
    }}}
    """

    def expand_macro(self, formatter, name, args):
        context = formatter.context
        resource = context.resource

        def get_page_text(formatter, page_resource):
            """Return a tuple of `(text, exists)` for the given page (resource)."""
            if page_resource.id == formatter.context.resource.id:
                return (formatter.source, True)
            else:
                page = WikiPage(self.env, page_resource)
                return (page.text, page.exists)

        out = StringIO()
        if not args: args = ''
        args = [x.strip() for x in args.split(',')]
        for pagename in args:
            page_resource = resource(id=pagename)
            if not 'WIKI_VIEW' in context.perm(page_resource):
                # No access to the page, so should not be included
                continue
            text, exists = get_page_text(formatter, page_resource)
            if not exists:
                out.write(system_message('Error: Page %s does not exist' %
                                     page_resource.id))
            else:
                ctx = formatter.context(page_resource)
                Formatter(self.env, ctx).format(text, out)
            out.write('<hr>')

        return Markup(out.getvalue())
